import json
from pathlib import Path

import ibm_watsonx_data_integration.services.datastage.models.flow_json_model as models
from ibm_watsonx_data_integration.services.datastage.codegen.code_generator import (
    # BuildStageCodeGenerator,
    ConnectionCodeGenerator,
    # CustomStageCodeGenerator,
    # DataDefinitionCodeGenerator,
    FlowCodeGenerator,
    # JobSettingsCodeGenerator,
    MasterCodeGenerator,
    # ParamSetCodeGenerator,
    # SubflowCodeGenerator,
    # WrappedStageCodeGenerator,
)
from ibm_watsonx_data_integration.services.datastage.codegen.dag_generator import (
    ConnectionGenerator,
    DAGGenerator,
)
from ibm_watsonx_data_integration.services.datastage.codegen.exporters.util import (
    _check_create_dir,
    _create_init_file,
    _format_code,
    _generate_flow_name,
)
from ibm_watsonx_data_integration.services.datastage.codegen.importers import ZipImporter


def _get_json_type(path: str):
    if path.startswith("data_intg_subflow"):
        return "subflow"
    elif path.startswith("data_intg_flow"):
        return "flow"
    elif path.startswith("parameter_set"):
        return "parameter_set"
    elif path.startswith("connection"):
        return "connection"
    elif path.startswith("job"):
        return "job"
    elif path.startswith("px_executables"):
        return "executables"
    elif path.startswith("ENCRYPTED"):
        return "encrypted"
    else:
        return ""


def _get_use_paramsets(paramsets: list):
    use_ps = ""
    for paramset in paramsets:
        use_ps += f"flow.use_paramset({paramset})\n"
    return use_ps


class MultiFileExporter:
    def __init__(
        self,
        zip_importer: ZipImporter,
        output_path: str | None,
        *,
        offline: bool = False,
        include_run: bool = True,
        print_logs: bool = True,
        include_create: bool = False,
        use_flow_name: bool = True,
    ):
        self.zip_importer = zip_importer
        if output_path is not None and output_path.strip() != "":
            self.output_path = str(Path(output_path).absolute())
            self.write_to_output = True
        else:
            self.output_path = ""
            self.write_to_output = False
        self.offline = offline
        self.include_run = include_run
        self.print_logs = print_logs
        self.include_create = include_create
        self.use_flow_name = use_flow_name

    def run(self):
        generated_code = {}

        master_gen = MasterCodeGenerator()
        imports: list[str] = []
        paramsets: list[str] = []
        job_vars: list[str] = []

        if self.write_to_output:
            _check_create_dir(self.output_path)
        else:
            self.output_path = ""  # Use relative path in returned code

        # paramset_path = Path(self.output_path) / "parameter_set"
        connection_path = Path(self.output_path) / "connection"
        # data_definition_path = Path(self.output_path) / "data_definition"
        job_path = Path(self.output_path) / "job"
        # build_stage_path = Path(self.output_path) / "data_intg_build_stage"
        # wrapped_stage_path = Path(self.output_path) / "data_intg_wrapped_stage"
        # custom_stage_path = Path(self.output_path) / "data_intg_custom_stage"
        # custom_stage_attachments_path = Path(self.output_path) / "custom_stage_library"
        # subflow_path = Path(self.output_path) / "data_intg_subflow"
        flow_path = Path(self.output_path) / "data_intg_flow"

        if self.write_to_output:  # Create and initialize output paths
            # if self.zip_importer.paramsets:
            #     _check_create_dir(str(paramset_path))
            #     _create_init_file(str(paramset_path))
            if self.zip_importer.connections:
                _check_create_dir(str(connection_path))
                _create_init_file(str(connection_path))
            # if self.zip_importer.data_definitions:
            #     _check_create_dir(str(data_definition_path))
            #     _create_init_file(str(data_definition_path))
            if self.zip_importer.jobs:
                _check_create_dir(str(job_path))
                _create_init_file(str(job_path))
            # if self.zip_importer.subflows:
            #     _check_create_dir(str(subflow_path))
            #     _create_init_file(str(subflow_path))
            # if self.zip_importer.build_stages:
            #     _check_create_dir(str(build_stage_path))
            #     _create_init_file(str(build_stage_path))
            # if self.zip_importer.wrapped_stages:
            #     _check_create_dir(str(wrapped_stage_path))
            #     _create_init_file(str(wrapped_stage_path))
            # if self.zip_importer.custom_stage_attachments:
            #     _check_create_dir(str(custom_stage_attachments_path))
            # if self.zip_importer.custom_stages:
            #     _check_create_dir(str(custom_stage_path))
            #     _create_init_file(str(custom_stage_path))
            if self.zip_importer.flows:
                _check_create_dir(str(flow_path))

        # for f_info, f_content in self.zip_importer.paramsets:
        #     mod_name = f_info.filename.split("/")[-1].replace(".json", "")
        #     imports.append(f"from parameter_set.{mod_name} import *")
        #     json_data = json.loads(f_content)["parameter_set"]
        #     param_set = ParameterSet.from_dict(json_data)
        #     ps_code_gen = ParamSetCodeGenerator(param_set, master_gen)
        #     ps_code = ps_code_gen.generate_code()
        #     formatted = _format_code(ps_code)
        #     var_name = master_gen.get_object_var(param_set)
        #     paramsets.append(var_name)
        #     # Save parameter set code
        #     file_name = paramset_path / (f_info.filename.split("/")[-1].replace("json", "py"))
        #     file_contents = ps_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        for f_info, f_content in self.zip_importer.connections:
            json_data = json.loads(f_content)
            conn_gen = ConnectionGenerator(json_data)
            conn_model = conn_gen.create_connection_model()
            conn_code_gen = ConnectionCodeGenerator(conn_model, master_gen)
            conn_code = conn_code_gen.generate_code()
            formatted = _format_code(conn_code)
            mod_name = master_gen.get_object_var(conn_model)
            imports.append(f"from connection.{mod_name} import *")
            # Save connections code
            file_name = connection_path / (mod_name + ".py")
            file_contents = conn_code_gen.get_imports() + f"\n\n{formatted}"
            generated_code[str(file_name)] = file_contents
            if self.write_to_output:
                with open(file_name, "w") as f:
                    f.write(file_contents)

        # for f_info, f_content in self.zip_importer.data_definitions:
        #     json_data = json.loads(f_content)
        #     data_def = DataDefinition.from_dict(json_data)
        #     data_def_code_gen = DataDefinitionCodeGenerator(data_def, master_gen)
        #     data_def_code = data_def_code_gen.generate_code()
        #     formatted = _format_code(data_def_code)
        #     # Save data definition code
        #     file_name = data_definition_path / (f_info.filename.split("/")[-1].replace("json", "py"))
        #     file_contents = data_def_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        # for f_info, f_content in self.zip_importer.jobs:
        #     mod_name = f_info.filename.split("/")[-1].replace(".json", "").replace(" ", "_").replace(".", "_")
        #     imports.append(f"from job.{mod_name} import *")
        #     job_json = json.loads(f_content)["entity"]["job"]
        #     job_gen = JobGenerator(job_json)
        #     job_settings = job_gen.create_job_model()
        #     job_code_gen = JobSettingsCodeGenerator(job_settings, master_gen)
        #     job_code = job_code_gen.generate_code()
        #     formatted = _format_code(job_code)
        #     job_var = master_gen.get_object_var(job_settings)
        #     job_vars.append(job_var)
        #     # Save job code
        #     file_name = job_path / (mod_name + ".py")
        #     file_contents = job_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        # for f_info, f_content in self.zip_importer.subflows:
        #     subflow_json = json.loads(f_content)
        #     subflow_name = subflow_json["name"] if "name" in subflow_json else f_info.filename.split("/")[-1].replace(".json", "")
        #     subflow_model = models.Flow(**subflow_json)
        #     subflow_dag_gen = DAGGenerator(subflow_model)
        #     dag = subflow_dag_gen.generate()._dag
        #     subflow = Subflow(dag=dag, name=subflow_name, is_local=False)
        #     master_gen.subflows[subflow_name] = subflow
        #     subflow_code_gen = SubflowCodeGenerator(
        #         subflow=subflow,
        #         master_gen=master_gen,
        #         offline=self.offline,
        #         zip_structure=True,
        #     )
        #     subflow_code = subflow_code_gen.generate_all()
        #     formatted = _format_code(subflow_code)
        #     mod_name = master_gen.get_object_var(subflow)
        #     imports.append(f"from data_intg_subflow.{mod_name} import *")
        #     # Save subflow code
        #     file_name = subflow_path / (mod_name + ".py")
        #     file_contents = subflow_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        # for f_info, f_content in self.zip_importer.build_stages:
        #     build_stage_json = json.loads(f_content)
        #     build_stage_name = (
        #         build_stage_json["name"] if "name" in build_stage_json else f_info.filename.split("/")[-1].replace(".json", "")
        #     )
        #     build_stage_model = BuildStage.from_dict(build_stage_json)
        #     build_stage_code_gen = BuildStageCodeGenerator(asset=build_stage_model, master_gen=master_gen)
        #     build_stage_code = build_stage_code_gen.generate_code()
        #     formatted = _format_code(build_stage_code)
        #     mod_name = master_gen.get_object_var(build_stage_model)
        #     build_stages[build_stage_name] = mod_name
        #     imports.append(f"from data_intg_build_stage.{mod_name} import *")
        #     # Save build stage code
        #     file_name = build_stage_path / (mod_name + ".py")
        #     file_contents = build_stage_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        # for f_info, f_content in self.zip_importer.wrapped_stages:
        #     wrapped_stage_json = json.loads(f_content)
        #     wrapped_stage_name = (
        #         wrapped_stage_json["name"] if "name" in wrapped_stage_json else f_info.filename.split("/")[-1].replace(".json", "")
        #     )
        #     wrapped_stage_model = WrappedStage.from_dict(wrapped_stage_json)
        #     wrapped_stage_code_gen = WrappedStageCodeGenerator(asset=wrapped_stage_model, master_gen=master_gen)
        #     wrapped_stage_code = wrapped_stage_code_gen.generate_code()
        #     formatted = _format_code(wrapped_stage_code)
        #     mod_name = master_gen.get_object_var(wrapped_stage_model)
        #     wrapped_stages[wrapped_stage_name] = mod_name
        #     imports.append(f"from data_intg_wrapped_stage.{mod_name} import *")
        #     # Save wrapped stage code
        #     file_name = wrapped_stage_path / (mod_name + ".py")
        #     file_contents = wrapped_stage_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        # for f_info, f_content in self.zip_importer.custom_stage_attachments:
        #     attachment_path = custom_stage_attachments_path / (Path(f_info.filename).stem + ".so")
        #     # Save custom stage attachments
        #     file_contents = str(base64.b64encode(f_content))[2:-1]
        #     generated_code[str(attachment_path)] = file_contents
        #     if self.write_to_output:
        #         with open(attachment_path, "w") as f:
        #             f.write(file_contents)

        #     custom_stage_paths[Path(f_info.filename).stem] = attachment_path

        # for f_info, f_content in self.zip_importer.custom_stages:
        #     # NEED TO ADD SOMETHING ABOUT FILE PATH
        #     custom_stage_json = json.loads(f_content)
        #     custom_stage_name = (
        #         custom_stage_json["name"] if "name" in custom_stage_json else f_info.filename.split("/")[-1].replace(".json", "")
        #     )
        #     for attachment in custom_stage_paths:
        #         if f"lib_{custom_stage_name}" == attachment:
        #             if "entity" in custom_stage_json:
        #                 custom_stage_json["entity"]["library_path"] = str(custom_stage_paths[attachment])
        #             else:
        #                 custom_stage_json["library_path"] = str(custom_stage_paths[attachment])
        #     custom_stage_model = CustomStage.from_dict(custom_stage_json)
        #     custom_stage_code_gen = CustomStageCodeGenerator(asset=custom_stage_model, master_gen=master_gen)
        #     custom_stage_code = custom_stage_code_gen.generate_code()
        #     formatted = _format_code(custom_stage_code)
        #     mod_name = master_gen.get_object_var(custom_stage_model)
        #     custom_stages[custom_stage_name] = mod_name
        #     imports.append(f"from data_intg_custom_stage.{mod_name} import *")
        #     # Save custom stage code
        #     file_name = custom_stage_path / (mod_name + ".py")
        #     file_contents = custom_stage_code_gen.get_imports() + f"\n\n{formatted}"
        #     generated_code[str(file_name)] = file_contents
        #     if self.write_to_output:
        #         with open(file_name, "w") as f:
        #             f.write(file_contents)

        for f_info, f_content in self.zip_importer.flows:
            flow_json = json.loads(f_content)
            try:
                flow_model = models.Flow(**flow_json)
            except Exception:
                try:
                    flow_json = flow_json["attachments"]
                    flow_model = models.Flow(**flow_json)
                except Exception:
                    try:
                        flow_json = flow_json[0]
                        flow_model = models.Flow(**flow_json)
                    except Exception as e:
                        raise ValueError(f"Bad flow json: {e}")

            dag_gen = DAGGenerator(flow_model)
            fc = dag_gen.generate()

            # for node in fc._dag.nodes():
            #     if isinstance(node, BuildStageStage):
            #         bs_name = node.configuration.op_name
            #         for build_stage in build_stages:
            #             if build_stage == bs_name:
            #                 node.build_stage = build_stages[build_stage]
            #     if isinstance(node, WrappedStageStage):
            #         ws_name = node.configuration.op_name
            #         for wrapped_stage in wrapped_stages:
            #             if wrapped_stage == ws_name:
            #                 node.wrapped_stage = wrapped_stages[wrapped_stage]
            #     if isinstance(node, CustomStageStage):
            #         cs_name = node.configuration.op_name
            #         for custom_stage in custom_stages:
            #             if custom_stage == cs_name:
            #                 node.custom_stage = custom_stages[custom_stage]

            code_gen = FlowCodeGenerator(fc, master_gen, self.offline, skip_subflows=True)
            code = code_gen.generate_all_zip()

            file_name = f_info.filename.split("/")[-1].replace("json", "py")
            if self.use_flow_name:
                flow_name = file_name.strip(".py")
            else:
                flow_name = _generate_flow_name()

            # Assemble flow code
            full_flow_path = flow_path / file_name
            file_contents = "\n".join(imports) + "\n" + code + f"\n{_get_use_paramsets(paramsets)}"
            if self.include_run:
                if len(job_vars) == 1:
                    file_contents += (
                        f'\n\nsdk.run_flow(flow=flow, flow_name="{flow_name}", job_settings={job_vars[0]}, print_logs={self.print_logs})'
                    )
                else:
                    file_contents += f'\n\njob_logs = sdk.run_flow(flow=flow, flow_name="{flow_name}", print_logs={self.print_logs})'
            if self.include_create:
                file_contents += f'\n\nsdk.create_flow(flow=flow, flow_name="{flow_name}")'

            # Save flow code
            generated_code[str(file_name)] = file_contents
            if self.write_to_output:
                with open(full_flow_path, "w") as f:
                    f.write(file_contents)

        return generated_code



from ibm_watsonx_data_integration.services.datastage.models.enums import (
    AZURE_FILE_STORAGE,
    AZURE_FILE_STORAGE_CONNECTION,
)
from ibm_watsonx_data_integration.services.datastage.models.flow.datastage_flow import DataStageFlow
from ibm_watsonx_data_integration.services.datastage.tests.integration.util import AbstractFlowRunTest
from os import environ
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzureFileStorageConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azurefilestorage_peek"

    def get_fc(self, project: Project):
        fc = DataStageFlow(project=project, name=self.get_flow_name(), description="", flow_type="datastage")

        # Stages
        _onpremcsv_1_ = fc.add_stage("Microsoft Azure File Storage", "onpremcsv_1")
        _onpremcsv_1_.configuration.ds_file_format = AZURE_FILE_STORAGE.DSFileFormat.comma_separated_value_csv
        _onpremcsv_1_.configuration.ds_file_name = "onprem.csv"
        _onpremcsv_1_.configuration.escape_character = AZURE_FILE_STORAGE.EscapeCharacter.double_quote
        _onpremcsv_1_.configuration.file_name = "onprem.csv"
        _onpremcsv_1_.configuration.file_name_source = "onprem.csv"
        _onpremcsv_1_.configuration.file_share = "test"
        _onpremcsv_1_.configuration.file_share_source = "test"
        _onpremcsv_1_.configuration.first_line_is_header = True
        _onpremcsv_1_.configuration.first_row_is_header = True
        _onpremcsv_1_.configuration.infer_as_varchar = False
        _onpremcsv_1_.configuration.infer_schema = False
        _onpremcsv_1_.configuration.quote_character = AZURE_FILE_STORAGE.QuoteCharacter.double_quote

        _onpremcsv_1_.configuration.connection.authentication_method = (
            AZURE_FILE_STORAGE_CONNECTION.AuthMethod.connection_string
        )
        _onpremcsv_1_.configuration.connection.connection_string = environ["AZUREFILESTORAGE_CONN_STRING"]

        _peek_2_ = fc.add_stage("Peek", "Peek_2")

        # Graph
        onpremcsv_1_schema = _onpremcsv_1_.connect_output_to(_peek_2_).set_name("Link_3").create_schema()

        # Schemas
        onpremcsv_1_schema.add_field("VARCHAR", "aaaaaa").nullable().length(1024)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import AZURE_FILE_STORAGE_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    connection_string = "connection_string"
    entra_id = "entra_id"
    entra_id_user = "entra_id_user"


class AzureFileStorageConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "2a7b4fa1-c770-4807-8871-a3c5def5aa2d"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    authentication_method: AZURE_FILE_STORAGE_CONNECTION.AuthMethod | None = Field(
        AZURE_FILE_STORAGE_CONNECTION.AuthMethod.connection_string, alias="auth_method"
    )
    client_id: str = Field(None, alias="client_id")
    client_secret: str = Field(None, alias="client_secret")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    connection_string: str = Field(None, alias="connection_string")
    container: str | None = Field(None, alias="container")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    password: str = Field(None, alias="password")
    storage_account_url: str = Field(None, alias="storage_account_url")
    tenant_id: str = Field(None, alias="tenant_id")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("tenant_id")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "entra_id"
                        )
                        or (self.authentication_method == "entra_id")
                    )
                )
            )
            else exclude.add("tenant_id")
        )
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "entra_id_user"
                        )
                        or (self.authentication_method == "entra_id_user")
                    )
                )
            )
            else exclude.add("password")
        )
        include.add("authentication_method") if (not self.defer_credentials) else exclude.add("authentication_method")
        (
            include.add("storage_account_url")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "entra_id"
                                )
                                or (self.authentication_method == "entra_id")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "entra_id_user"
                            )
                            or (self.authentication_method == "entra_id_user")
                        )
                    )
                )
            )
            else exclude.add("storage_account_url")
        )
        (
            include.add("client_secret")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "entra_id"
                        )
                        or (self.authentication_method == "entra_id")
                    )
                )
            )
            else exclude.add("client_secret")
        )
        (
            include.add("connection_string")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "connection_string"
                        )
                        or (self.authentication_method == "connection_string")
                    )
                )
            )
            else exclude.add("connection_string")
        )
        (
            include.add("client_id")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "entra_id"
                                )
                                or (self.authentication_method == "entra_id")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "entra_id_user"
                            )
                            or (self.authentication_method == "entra_id_user")
                        )
                    )
                )
            )
            else exclude.add("client_id")
        )
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "entra_id_user"
                        )
                        or (self.authentication_method == "entra_id_user")
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")

        (
            include.add("storage_account_url")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "entra_id" in str(self.authentication_method.value)
                    )
                    or ("entra_id" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "entra_id_user" in str(self.authentication_method.value)
                    )
                    or ("entra_id_user" in str(self.authentication_method))
                )
            )
            else exclude.add("storage_account_url")
        )
        include.add("authentication_method") if (not self.defer_credentials) else exclude.add("authentication_method")
        (
            include.add("tenant_id")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (hasattr(self.authentication_method, "value") and self.authentication_method.value == "entra_id")
                    or (self.authentication_method == "entra_id")
                )
            )
            else exclude.add("tenant_id")
        )
        (
            include.add("client_id")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "entra_id" in str(self.authentication_method.value)
                    )
                    or ("entra_id" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "entra_id_user" in str(self.authentication_method.value)
                    )
                    or ("entra_id_user" in str(self.authentication_method))
                )
            )
            else exclude.add("client_id")
        )
        (
            include.add("connection_string")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value == "connection_string"
                    )
                    or (self.authentication_method == "connection_string")
                )
            )
            else exclude.add("connection_string")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value == "entra_id_user"
                    )
                    or (self.authentication_method == "entra_id_user")
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value == "entra_id_user"
                    )
                    or (self.authentication_method == "entra_id_user")
                )
            )
            else exclude.add("username")
        )
        (
            include.add("client_secret")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (hasattr(self.authentication_method, "value") and self.authentication_method.value == "entra_id")
                    or (self.authentication_method == "entra_id")
                )
            )
            else exclude.add("client_secret")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



import warnings
from ibm_watsonx_data_integration.services.datastage.models.base_stage import BaseStage
from ibm_watsonx_data_integration.services.datastage.models.connections.bigsql_connection import BigsqlConn
from ibm_watsonx_data_integration.services.datastage.models.enums import BIGSQL
from pydantic import Field
from typing import ClassVar


class bigsql(BaseStage):

    op_name: ClassVar[str] = "bigsql"
    node_type: ClassVar[str] = "binding"
    image: ClassVar[str] = "/data-intg/flows/graphics/palette/bigsql.svg"
    label: ClassVar[str] = "IBM Db2 Big SQL"
    runtime_column_propagation: bool = Field(False, alias="runtime_column_propagation")
    connection: BigsqlConn = BigsqlConn()
    add_procedure_return_value_to_schema: bool | None = Field(False, alias="add_proccode_column")
    batch_size: int | None = Field(2000, alias="batch_size")
    buf_free_run_ronly: int | None = Field(50, alias="buf_free_run_ronly")
    buf_mode_ronly: BIGSQL.BufModeRonly | None = Field(BIGSQL.BufModeRonly.default, alias="buf_mode_ronly")
    buffer_free_run_percent: int | None = Field(50, alias="buf_free_run")
    buffering_mode: BIGSQL.BufferingMode | None = Field(BIGSQL.BufferingMode.default, alias="buf_mode")
    byte_limit: str | None = Field(None, alias="byte_limit")
    call_procedure_statement: str | None = Field(None, alias="call_statement")
    collecting: BIGSQL.Collecting | None = Field(BIGSQL.Collecting.auto, alias="coll_type")
    column_metadata_change_propagation: bool | None = Field(None, alias="auto_column_propagation")
    combinability_mode: BIGSQL.CombinabilityMode | None = Field(BIGSQL.CombinabilityMode.auto, alias="combinability")
    create_data_asset: bool | None = Field(False, alias="registerDataAsset")
    create_statement: str | None = Field(None, alias="create_statement")
    credentials_input_method_ssl: BIGSQL.CredentialsInputMethodSsl | None = Field(
        BIGSQL.CredentialsInputMethodSsl.enter_credentials_manually, alias="credentials_input_method_ssl"
    )
    current_output_link_type: str = Field("PRIMARY", alias="currentOutputLinkType")
    data_asset_name: str = Field(None, alias="dataAssetName")
    db2_database_name: str | None = Field(None, alias="part_client_dbname")
    db2_instance_name: str | None = Field(None, alias="part_client_instance")
    db2_source_connection_required: str | None = Field("", alias="part_dbconnection")
    db2_table_name: str | None = Field(None, alias="part_table")
    decimal_rounding_mode: BIGSQL.DecimalRoundingMode | None = Field(
        BIGSQL.DecimalRoundingMode.floor, alias="decimal_rounding_mode"
    )
    default_maximum_length_for_columns: int | None = Field(20000, alias="default_max_string_binary_precision")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    disk_write_inc_ronly: int | None = Field(1048576, alias="disk_write_inc_ronly")
    disk_write_increment_bytes: int | None = Field(1048576, alias="disk_write_inc")
    ds_java_heap_size: int | None = Field(256, alias="_java._heap_size")
    enable_after_sql: str | None = Field("", alias="before_after.after")
    enable_after_sql_node: str | None = Field("", alias="before_after.after_node")
    enable_before_sql: str | None = Field("", alias="before_after.before")
    enable_before_sql_node: str | None = Field("", alias="before_after.before_node")
    enable_flow_acp_control: bool = Field(True, alias="enableFlowAcpControl")
    enable_schemaless_design: bool = Field(False, alias="enableSchemalessDesign")
    error_warning: str | None = Field(None, alias="error_warning")
    execute_procedure_for_each_row: bool | None = Field(True, alias="call_each_row")
    execution_mode: BIGSQL.ExecutionMode | None = Field(BIGSQL.ExecutionMode.default_par, alias="execmode")
    existing_table_action: BIGSQL.ExistingTableAction | None = Field(
        BIGSQL.ExistingTableAction.append, alias="existing_table_action"
    )
    fail_on_error_after_sql: bool | None = Field(True, alias="before_after.after.fail_on_error")
    fail_on_error_after_sql_node: bool | None = Field(True, alias="before_after.after_node.fail_on_error")
    fail_on_error_before_sql: bool | None = Field(True, alias="before_after.before.fail_on_error")
    fail_on_error_before_sql_node: bool | None = Field(True, alias="before_after.before_node.fail_on_error")
    fatal_error: str | None = Field(None, alias="error_fatal")
    flow_dirty: str | None = Field("false", alias="flow_dirty")
    forward_row_data: bool | None = Field(False, alias="forward_row_data")
    generate_unicode_type_columns: bool | None = Field(False, alias="generate_unicode_columns")
    has_reject_output: bool | None = Field(False, alias="has_reject_output")
    hide: bool | None = Field(False, alias="hide")
    infer_schema: bool | None = Field(True, alias="rcp")
    input_count: int | None = Field(0, alias="input_count")
    input_link_description: list | None = Field("", alias="inputLinkDescription")
    input_method: BIGSQL.InputMethod | None = Field(
        BIGSQL.InputMethod.enter_credentials_manually, alias="credentials_input_method"
    )
    inputcol_properties: list | None = Field([], alias="inputcolProperties")
    key_cols_coll: list | None = Field([], alias="keyColsColl")
    key_cols_coll_ordered: list | None = Field([], alias="keyColsCollOrdered")
    key_cols_coll_robin: list | None = Field([], alias="keyColsCollRobin")
    key_cols_none: list | None = Field([], alias="keyColsNone")
    key_cols_part: list | None = Field([], alias="keyColsPart")
    key_column_names: str | None = Field(None, alias="key_column_names")
    max_mem_buf_size_ronly: int | None = Field(3145728, alias="max_mem_buf_size_ronly")
    maximum_memory_buffer_size_bytes: int | None = Field(3145728, alias="max_mem_buf_size")
    output_acp_should_hide: bool = Field(True, alias="outputAcpShouldHide")
    output_count: int | None = Field(1, alias="output_count")
    output_link_description: list | None = Field("", alias="outputLinkDescription")
    outputcol_properties: list | None = Field([], alias="outputcolProperties")
    part_stable_coll: bool | None = Field(False, alias="part_stable_coll")
    part_stable_ordered: bool | None = Field(False, alias="part_stable_ordered")
    part_stable_roundrobin_coll: bool | None = Field(False, alias="part_stable_roundrobin_coll")
    part_unique_coll: bool | None = Field(False, alias="part_unique_coll")
    part_unique_ordered: bool | None = Field(False, alias="part_unique_ordered")
    part_unique_roundrobin_coll: bool | None = Field(False, alias="part_unique_roundrobin_coll")
    partition_type: BIGSQL.PartitionType | None = Field(BIGSQL.PartitionType.auto, alias="part_type")
    perform_sort: bool | None = Field(False, alias="perform_sort")
    perform_sort_coll: bool | None = Field(False, alias="perform_sort_coll")
    perform_sort_modulus: bool | None = Field(False, alias="perform_sort_modulus")
    preserve_partitioning: BIGSQL.PreservePartitioning | None = Field(
        BIGSQL.PreservePartitioning.default_propagate, alias="preserve"
    )
    proc_param_properties: list | None = Field([], alias="procParamProperties")
    push_filters: str | None = Field(None, alias="push_filters")
    pushed_filters: str | None = Field(None, alias="pushed_filters")
    query_timeout: int | None = Field(None, alias="query_timeout")
    queue_upper_bound_size_bytes: int | None = Field(0, alias="queue_upper_size")
    queue_upper_size_ronly: int | None = Field(0, alias="queue_upper_size_ronly")
    read_method: BIGSQL.ReadMethod | None = Field(BIGSQL.ReadMethod.general, alias="read_mode")
    reject_condition_row_is_rejected: bool | None = Field(False, alias="reject_condition_row_is_rejected")
    reject_data_element_errorcode: bool | None = Field(False, alias="reject_data_element_errorcode")
    reject_data_element_errortext: bool | None = Field(False, alias="reject_data_element_errortext")
    reject_from_link: int | None = Field(None, alias="reject_from_link")
    reject_number: int | None = Field(None, alias="reject_number")
    reject_threshold: int | None = Field(None, alias="reject_threshold")
    reject_uses: BIGSQL.RejectUses | None = Field(BIGSQL.RejectUses.rows, alias="reject_uses")
    rejected_filters: str | None = Field(None, alias="rejected_filters")
    row_limit: int | None = Field(None, alias="row_limit")
    runtime_column_propagation: bool | None = Field(None, alias="runtime_column_propagation")
    schema_name: str | None = Field(None, alias="schema_name")
    select_statement: str = Field(None, alias="select_statement")
    show_coll_type: int | None = Field(0, alias="showCollType")
    show_part_type: int | None = Field(1, alias="showPartType")
    show_sort_options: int | None = Field(0, alias="showSortOptions")
    sort_instructions: str | None = Field("", alias="sortInstructions")
    sort_instructions_text: str | None = Field("", alias="sortInstructionsText")
    sorting_key: BIGSQL.KeyColSelect | None = Field(BIGSQL.KeyColSelect.default, alias="keyColSelect")
    stable: bool | None = Field(None, alias="part_stable")
    stage_description: list | None = Field("", alias="stageDescription")
    static_statement: str = Field(None, alias="static_statement")
    stored_procedure_name: str | None = Field(None, alias="procedure_name")
    table_action: BIGSQL.TableAction | None = Field(BIGSQL.TableAction.append, alias="table_action")
    table_name: str = Field(None, alias="table_name")
    transform: str | None = Field("false", alias="transform")
    unique: bool | None = Field(None, alias="part_unique")
    update_statement: str | None = Field(None, alias="update_statement")
    user_defined_function: bool | None = Field(None, alias="user_defined_function")
    write_mode: BIGSQL.WriteMode | None = Field(BIGSQL.WriteMode.insert, alias="write_mode")

    def validate_parameters(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        (
            include.add("preserve_partitioning")
            if (self.output_count and self.output_count > 0)
            else exclude.add("preserve_partitioning")
        )
        (
            include.add("reject_uses")
            if (self.reject_from_link and self.reject_from_link > -1)
            else exclude.add("reject_uses")
        )
        (
            include.add("reject_number")
            if (self.reject_from_link and self.reject_from_link > -1)
            else exclude.add("reject_number")
        )
        (
            include.add("reject_data_element_errorcode")
            if (self.reject_from_link and self.reject_from_link > -1)
            else exclude.add("reject_data_element_errorcode")
        )
        (
            include.add("reject_data_element_errortext")
            if (self.reject_from_link and self.reject_from_link > -1)
            else exclude.add("reject_data_element_errortext")
        )
        include.add("reject_threshold") if (self.reject_uses == "percent") else exclude.add("reject_threshold")
        return include, exclude

    def validate_source(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        (
            include.add("call_procedure_statement")
            if (
                ((not self.table_name))
                and (not self.select_statement)
                and (not self.stored_procedure_name)
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                            or (self.read_method == "call_statement")
                        )

                )
            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("row_limit")
            if (

                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value != "call")
                                or (self.read_method != "call")
                            )
                        )
                    )
                    and (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value != "call_statement")
                                or (self.read_method != "call_statement")
                            )

                    )

            )
            else exclude.add("row_limit")
        )
        (
            include.add("select_statement")
            if (
                ((not self.schema_name))
                and (not self.table_name)
                and (not self.stored_procedure_name)
                and (not self.call_procedure_statement)
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value == "select")
                            or (self.read_method == "select")
                        )

                )
            )
            else exclude.add("select_statement")
        )
        (
            include.add("schema_name")
            if (
                ((not self.select_statement))
                and (not self.call_procedure_statement)
                and (
                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "general")
                                or (self.read_method == "general")
                            )

                    )
                )
            )
            else exclude.add("schema_name")
        )
        (
            include.add("table_name")
            if (
                ((not self.select_statement))
                and (not self.stored_procedure_name)
                and (not self.call_procedure_statement)
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value == "general")
                            or (self.read_method == "general")
                        )

                )
            )
            else exclude.add("table_name")
        )
        (
            include.add("user_defined_function")
            if (

                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                or (self.read_method == "call_statement")
                            )

                    )

            )
            else exclude.add("user_defined_function")
        )
        (
            include.add("add_procedure_return_value_to_schema")
            if (
                ((not self.select_statement))
                and (not self.table_name)
                and (
                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                or (self.read_method == "call_statement")
                            )

                    )
                )
            )
            else exclude.add("add_procedure_return_value_to_schema")
        )
        (
            include.add("stored_procedure_name")
            if (
                ((not self.call_procedure_statement))
                and (not self.select_statement)
                and (not self.table_name)
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value == "call")
                            or (self.read_method == "call")
                        )

                )
            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("forward_row_data")
            if (
                ((not self.select_statement))
                and (not self.table_name)
                and (
                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                or (self.read_method == "call_statement")
                            )

                    )
                )
            )
            else exclude.add("forward_row_data")
        )
        (
            include.add("fatal_error")
            if (
                ((not self.select_statement))
                and (not self.table_name)
                and (
                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                or (self.read_method == "call_statement")
                            )

                    )
                )
            )
            else exclude.add("fatal_error")
        )
        (
            include.add("error_warning")
            if (
                ((not self.select_statement))
                and (not self.table_name)
                and (
                    (
                        (
                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                or (self.read_method == "call")
                            )
                        )
                    )
                    or (

                            self.read_method
                            and (
                                (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                or (self.read_method == "call_statement")
                            )

                    )
                )
            )
            else exclude.add("error_warning")
        )
        (
            include.add("call_procedure_statement")
            if (

                    (((not self.table_name)) or ((self.table_name and "#" in str(self.table_name))))
                    and (
                        ((not self.select_statement)) or (self.select_statement and "#" in str(self.select_statement))
                    )
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("user_defined_function")
            if (


                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )


            )
            else exclude.add("user_defined_function")
        )
        (
            include.add("add_procedure_return_value_to_schema")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("add_procedure_return_value_to_schema")
        )
        (
            include.add("stored_procedure_name")
            if (

                    (
                        ((not self.call_procedure_statement))
                        or ((self.call_procedure_statement and "#" in str(self.call_procedure_statement)))
                    )
                    and (
                        ((not self.select_statement)) or (self.select_statement and "#" in str(self.select_statement))
                    )
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("forward_row_data")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("forward_row_data")
        )
        (
            include.add("fatal_error")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("fatal_error")
        )
        (
            include.add("row_limit")
            if (



                            (
                                (
                                    (
                                        self.read_method
                                        and (
                                            (hasattr(self.read_method, "value") and self.read_method.value != "call")
                                            or (self.read_method != "call")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.read_method
                                        and (
                                            (
                                                hasattr(self.read_method, "value")
                                                and self.read_method.value
                                                and "#" in str(self.read_method.value)
                                            )
                                            or ("#" in str(self.read_method))
                                        )
                                    )
                                )
                            )
                            and (
                                (
                                    (
                                        self.read_method
                                        and (
                                            (
                                                hasattr(self.read_method, "value")
                                                and self.read_method.value != "call_statement"
                                            )
                                            or (self.read_method != "call_statement")
                                        )
                                    )
                                )
                                or (

                                        self.read_method
                                        and (
                                            (
                                                hasattr(self.read_method, "value")
                                                and self.read_method.value
                                                and "#" in str(self.read_method.value)
                                            )
                                            or ("#" in str(self.read_method))
                                        )

                                )
                            )



            )
            else exclude.add("row_limit")
        )
        (
            include.add("select_statement")
            if (

                    (((not self.schema_name)) or ((self.schema_name and "#" in str(self.schema_name))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "select")
                                    or (self.read_method == "select")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("select_statement")
        )
        (
            include.add("schema_name")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "general")
                                    or (self.read_method == "general")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("schema_name")
        )
        (
            include.add("table_name")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "general")
                                    or (self.read_method == "general")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("table_name")
        )
        (
            include.add("error_warning")
            if (

                    (((not self.select_statement)) or ((self.select_statement and "#" in str(self.select_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        (
                            (
                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                                    or (self.read_method == "call")
                                )
                            )
                        )
                        or (

                                self.read_method
                                and (
                                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                                    or (self.read_method == "call_statement")
                                )

                        )
                        or (

                                self.read_method
                                and (
                                    (
                                        hasattr(self.read_method, "value")
                                        and self.read_method.value
                                        and "#" in str(self.read_method.value)
                                    )
                                    or ("#" in str(self.read_method))
                                )

                        )
                    )

            )
            else exclude.add("error_warning")
        )

        (
            include.add("maximum_memory_buffer_size_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("maximum_memory_buffer_size_bytes")
        )
        (
            include.add("buffer_free_run_percent")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("buffer_free_run_percent")
        )
        (
            include.add("queue_upper_bound_size_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("queue_upper_bound_size_bytes")
        )
        (
            include.add("disk_write_increment_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("disk_write_increment_bytes")
        )
        (
            include.add("runtime_column_propagation")
            if (
                ((not self.enable_schemaless_design))
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value != "call")
                            or (self.read_method != "call")
                        )

                )
                and (

                        self.read_method
                        and (
                            (hasattr(self.read_method, "value") and self.read_method.value != "call_statement")
                            or (self.read_method != "call_statement")
                        )

                )
            )
            else exclude.add("runtime_column_propagation")
        )
        (
            include.add("column_metadata_change_propagation")
            if (not self.output_acp_should_hide)
            else exclude.add("column_metadata_change_propagation")
        )
        (
            include.add("max_mem_buf_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("max_mem_buf_size_ronly")
        )
        (
            include.add("buf_free_run_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("buf_free_run_ronly")
        )
        (
            include.add("queue_upper_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("queue_upper_size_ronly")
        )
        (
            include.add("disk_write_inc_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("disk_write_inc_ronly")
        )
        (
            include.add("stable")
            if (

                    ((self.show_part_type))
                    and (self.partition_type != "auto")
                    and (self.partition_type != "db2connector")
                    and (self.show_sort_options)

            )
            else exclude.add("stable")
        )
        (
            include.add("unique")
            if (

                    ((self.show_part_type))
                    and (self.partition_type != "auto")
                    and (self.partition_type != "db2connector")
                    and (self.show_sort_options)

            )
            else exclude.add("unique")
        )
        (
            include.add("key_cols_part")
            if (
                (
                    (
                        ((self.show_part_type))
                        and ((not self.show_coll_type))
                        and ((self.partition_type != "auto"))
                        and ((self.partition_type != "db2connector"))
                        and ((self.partition_type != "modulus"))
                    )
                )
                or (

                        ((self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.partition_type == "modulus")
                        and (self.perform_sort_modulus)

                )
            )
            else exclude.add("key_cols_part")
        )
        (
            include.add("db2_source_connection_required")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_source_connection_required")
        )
        (
            include.add("db2_database_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_database_name")
        )
        (
            include.add("db2_instance_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_instance_name")
        )
        (
            include.add("db2_table_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_table_name")
        )
        (
            include.add("perform_sort")
            if (

                    ((self.show_part_type))
                    and (((self.partition_type == "hash")) or (self.partition_type == "range"))

            )
            else exclude.add("perform_sort")
        )
        (
            include.add("perform_sort_modulus")
            if (((self.show_part_type)) and (self.partition_type == "modulus"))
            else exclude.add("perform_sort_modulus")
        )
        (
            include.add("sorting_key")
            if (

                    ((self.show_part_type))
                    and (self.partition_type == "modulus")
                    and (not self.perform_sort_modulus)

            )
            else exclude.add("sorting_key")
        )
        (
            include.add("sort_instructions")
            if (

                    ((self.show_part_type))
                    and (
                        ((self.partition_type == "db2part"))
                        or (self.partition_type == "entire")
                        or (self.partition_type == "random")
                        or (self.partition_type == "roundrobin")
                        or (self.partition_type == "same")
                    )

            )
            else exclude.add("sort_instructions")
        )
        (
            include.add("sort_instructions_text")
            if (

                    ((self.show_part_type))
                    and (
                        ((self.partition_type == "db2part"))
                        or (self.partition_type == "entire")
                        or (self.partition_type == "random")
                        or (self.partition_type == "roundrobin")
                        or (self.partition_type == "same")
                    )

            )
            else exclude.add("sort_instructions_text")
        )
        include.add("collecting") if (self.show_coll_type) else exclude.add("collecting")
        include.add("partition_type") if (self.show_part_type) else exclude.add("partition_type")
        (
            include.add("perform_sort_coll")
            if (
                (
                    (
                        ((self.show_coll_type))
                        and (
                            ((self.collecting == "ordered"))
                            or ((self.collecting == "roundrobin_coll"))
                            or ((self.collecting == "sortmerge"))
                        )
                    )
                )
                or (((not self.show_part_type)) and (not self.show_coll_type))
            )
            else exclude.add("perform_sort_coll")
        )
        (
            include.add("key_cols_coll")
            if (

                    ((self.show_coll_type))
                    and (not self.show_part_type)
                    and (self.collecting != "auto")
                    and (((self.collecting == "sortmerge")) or (self.perform_sort_coll))

            )
            else exclude.add("key_cols_coll")
        )
        (
            include.add("key_cols_none")
            if (

                    ((not self.show_part_type))
                    and (not self.show_coll_type)
                    and (self.perform_sort_coll)

            )
            else exclude.add("key_cols_none")
        )
        (
            include.add("part_stable_coll")
            if (

                    (((self.perform_sort_coll)))
                    and (
                        (
                            (
                                ((not self.show_part_type))
                                and ((self.show_coll_type))
                                and ((self.collecting != "auto"))
                                and ((self.show_sort_options))
                            )
                        )
                        or (

                                ((not self.show_part_type))
                                and (not self.show_coll_type)
                                and (self.show_sort_options)

                        )
                    )

            )
            else exclude.add("part_stable_coll")
        )
        (
            include.add("part_unique_coll")
            if (

                    (((self.perform_sort_coll)))
                    and (
                        (
                            (
                                ((not self.show_part_type))
                                and ((self.show_coll_type))
                                and ((self.collecting != "auto"))
                                and ((self.show_sort_options))
                            )
                        )
                        or (

                                ((not self.show_part_type))
                                and (not self.show_coll_type)
                                and (self.show_sort_options)

                        )
                    )

            )
            else exclude.add("part_unique_coll")
        )
        include.add("defer_credentials") if (()) else exclude.add("defer_credentials")
        (
            include.add("credentials_input_method_ssl")
            if ((()) and ((()) or (())))
            else exclude.add("credentials_input_method_ssl")
        )
        (
            include.add("schema_name")
            if (not self.select_statement)
            and (not self.call_procedure_statement)
            and (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "general" in str(self.read_method.value)
                    )
                    or ("general" in str(self.read_method))
                )
            )
            else exclude.add("schema_name")
        )
        (
            include.add("stored_procedure_name")
            if (not self.call_procedure_statement)
            and (not self.select_statement)
            and (not self.table_name)
            and (
                self.read_method
                and (
                    (hasattr(self.read_method, "value") and self.read_method.value == "call")
                    or (self.read_method == "call")
                )
            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("error_warning")
            if (not self.select_statement)
            and (not self.table_name)
            and (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" in str(self.read_method.value)
                    )
                    or ("call_statement" in str(self.read_method))
                )
            )
            else exclude.add("error_warning")
        )
        (
            include.add("table_name")
            if (not self.select_statement)
            and (not self.stored_procedure_name)
            and (not self.call_procedure_statement)
            and (
                self.read_method
                and (
                    (hasattr(self.read_method, "value") and self.read_method.value == "general")
                    or (self.read_method == "general")
                )
            )
            else exclude.add("table_name")
        )
        (
            include.add("select_statement")
            if (not self.schema_name)
            and (not self.table_name)
            and (not self.stored_procedure_name)
            and (not self.call_procedure_statement)
            and (
                self.read_method
                and (
                    (hasattr(self.read_method, "value") and self.read_method.value == "select")
                    or (self.read_method == "select")
                )
            )
            else exclude.add("select_statement")
        )
        (
            include.add("add_procedure_return_value_to_schema")
            if (not self.select_statement)
            and (not self.table_name)
            and (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" in str(self.read_method.value)
                    )
                    or ("call_statement" in str(self.read_method))
                )
            )
            else exclude.add("add_procedure_return_value_to_schema")
        )
        (
            include.add("forward_row_data")
            if (not self.select_statement)
            and (not self.table_name)
            and (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" in str(self.read_method.value)
                    )
                    or ("call_statement" in str(self.read_method))
                )
            )
            else exclude.add("forward_row_data")
        )
        (
            include.add("row_limit")
            if (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" not in str(self.read_method.value)
                    )
                    or ("call" not in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" not in str(self.read_method.value)
                    )
                    or ("call_statement" not in str(self.read_method))
                )
            )
            else exclude.add("row_limit")
        )
        (
            include.add("call_procedure_statement")
            if (not self.table_name)
            and (not self.select_statement)
            and (not self.stored_procedure_name)
            and (
                self.read_method
                and (
                    (hasattr(self.read_method, "value") and self.read_method.value == "call_statement")
                    or (self.read_method == "call_statement")
                )
            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("fatal_error")
            if (not self.select_statement)
            and (not self.table_name)
            and (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" in str(self.read_method.value)
                    )
                    or ("call_statement" in str(self.read_method))
                )
            )
            else exclude.add("fatal_error")
        )
        (
            include.add("user_defined_function")
            if (
                self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call" in str(self.read_method.value)
                    )
                    or ("call" in str(self.read_method))
                )
                and self.read_method
                and (
                    (
                        hasattr(self.read_method, "value")
                        and self.read_method.value
                        and "call_statement" in str(self.read_method.value)
                    )
                    or ("call_statement" in str(self.read_method))
                )
            )
            else exclude.add("user_defined_function")
        )
        return include, exclude

    def validate_target(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        (
            include.add("create_statement")
            if (
                ((not self.static_statement))
                and (not self.stored_procedure_name)
                and (not self.call_procedure_statement)
                and (
                    (
                        (
                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "call")
                                or (self.write_mode != "call")
                            )
                        )
                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "call_statement")
                                or (self.write_mode != "call_statement")
                            )

                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "static_statement")
                                or (self.write_mode != "static_statement")
                            )

                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "update_statement")
                                or (self.write_mode != "update_statement")
                            )

                    )
                )
            )
            else exclude.add("create_statement")
        )
        (
            include.add("update_statement")
            if (
                ((not self.static_statement))
                and (
                    (
                        (
                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value == "update_statement")
                                or (self.write_mode == "update_statement")
                            )
                        )
                    )
                    or (

                            self.write_mode
                            and (
                                (
                                    hasattr(self.write_mode, "value")
                                    and self.write_mode.value == "update_statement_table_action"
                                )
                                or (self.write_mode == "update_statement_table_action")
                            )

                    )
                )
            )
            else exclude.add("update_statement")
        )
        (
            include.add("schema_name")
            if (
                ((not self.static_statement))
                and (not self.call_procedure_statement)
                and (
                    (
                        (
                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "call_statement")
                                or (self.write_mode != "call_statement")
                            )
                        )
                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "static_statement")
                                or (self.write_mode != "static_statement")
                            )

                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "update_statement")
                                or (self.write_mode != "update_statement")
                            )

                    )
                )
            )
            else exclude.add("schema_name")
        )
        (
            include.add("table_name")
            if (
                ((not self.static_statement))
                and (not self.call_procedure_statement)
                and (
                    (
                        (
                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "call")
                                or (self.write_mode != "call")
                            )
                        )
                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "call_statement")
                                or (self.write_mode != "call_statement")
                            )

                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "static_statement")
                                or (self.write_mode != "static_statement")
                            )

                    )
                    and (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value != "update_statement")
                                or (self.write_mode != "update_statement")
                            )

                    )
                )
            )
            else exclude.add("table_name")
        )
        (
            include.add("key_column_names")
            if (
                (((not self.update_statement)) and ((not self.static_statement)))
                and (
                    (
                        (
                            (
                                self.existing_table_action
                                and (
                                    (
                                        hasattr(self.existing_table_action, "value")
                                        and self.existing_table_action.value == "append"
                                    )
                                    or (self.existing_table_action == "append")
                                )
                            )
                        )
                        or (
                            (
                                self.existing_table_action
                                and (
                                    (
                                        hasattr(self.existing_table_action, "value")
                                        and self.existing_table_action.value == "merge"
                                    )
                                    or (self.existing_table_action == "merge")
                                )
                            )
                        )
                        or (
                            (
                                self.existing_table_action
                                and (
                                    (
                                        hasattr(self.existing_table_action, "value")
                                        and self.existing_table_action.value == "update"
                                    )
                                    or (self.existing_table_action == "update")
                                )
                            )
                        )
                    )
                    or (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "insert")
                                    or (self.write_mode == "insert")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "merge")
                                    or (self.write_mode == "merge")
                                )

                        )
                        or (

                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "update")
                                    or (self.write_mode == "update")
                                )

                        )
                    )
                )
            )
            else exclude.add("key_column_names")
        )
        (
            include.add("table_action")
            if (
                (
                    (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "insert")
                                    or (self.write_mode == "insert")
                                )
                            )
                        )
                        or (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "merge")
                                    or (self.write_mode == "merge")
                                )
                            )
                        )
                        or (
                            (
                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value == "update_statement_table_action"
                                    )
                                    or (self.write_mode == "update_statement_table_action")
                                )
                            )
                        )
                    )
                )
                and (
                    ((not self.static_statement))
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value != "static_statement")
                                    or (self.write_mode != "static_statement")
                                )
                            )
                        )
                        and (

                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value != "update_statement")
                                    or (self.write_mode != "update_statement")
                                )

                        )
                    )
                )
            )
            else exclude.add("table_action")
        )
        (
            include.add("static_statement")
            if (
                ((not self.schema_name))
                and (not self.table_name)
                and (not self.update_statement)
                and (not self.stored_procedure_name)
                and (not self.call_procedure_statement)
                and (

                        self.write_mode
                        and (
                            (hasattr(self.write_mode, "value") and self.write_mode.value == "static_statement")
                            or (self.write_mode == "static_statement")
                        )

                )
            )
            else exclude.add("static_statement")
        )
        (
            include.add("call_procedure_statement")
            if (
                ((not self.static_statement))
                and (not self.schema_name)
                and (not self.table_name)
                and (not self.update_statement)
                and (not self.stored_procedure_name)
                and (

                        self.write_mode
                        and (
                            (hasattr(self.write_mode, "value") and self.write_mode.value == "call_statement")
                            or (self.write_mode == "call_statement")
                        )

                )
            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("stored_procedure_name")
            if (
                ((not self.static_statement))
                and (not self.table_name)
                and (not self.update_statement)
                and (not self.call_procedure_statement)
                and (

                        self.write_mode
                        and (
                            (hasattr(self.write_mode, "value") and self.write_mode.value == "call")
                            or (self.write_mode == "call")
                        )

                )
            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("batch_size")
            if (((not self.has_reject_output)) or (not self.has_reject_output))
            else exclude.add("batch_size")
        )
        (
            include.add("execute_procedure_for_each_row")
            if (
                ((not self.static_statement))
                and (not self.table_name)
                and (not self.update_statement)
                and (
                    (
                        (
                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value == "call")
                                or (self.write_mode == "call")
                            )
                        )
                    )
                    or (

                            self.write_mode
                            and (
                                (hasattr(self.write_mode, "value") and self.write_mode.value == "call_statement")
                                or (self.write_mode == "call_statement")
                            )

                    )
                )
            )
            else exclude.add("execute_procedure_for_each_row")
        )
        (
            include.add("call_procedure_statement")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (((not self.schema_name)) or (self.schema_name and "#" in str(self.schema_name)))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        ((not self.update_statement)) or (self.update_statement and "#" in str(self.update_statement))
                    )
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call_statement")
                                    or (self.write_mode == "call_statement")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value
                                        and "#" in str(self.write_mode.value)
                                    )
                                    or ("#" in str(self.write_mode))
                                )

                        )
                    )

            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("stored_procedure_name")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        ((not self.update_statement)) or (self.update_statement and "#" in str(self.update_statement))
                    )
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call")
                                    or (self.write_mode == "call")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value
                                        and "#" in str(self.write_mode.value)
                                    )
                                    or ("#" in str(self.write_mode))
                                )

                        )
                    )

            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("create_statement")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (

                            (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (hasattr(self.write_mode, "value") and self.write_mode.value != "call")
                                            or (self.write_mode != "call")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )
                                    )
                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "call_statement"
                                            )
                                            or (self.write_mode != "call_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "static_statement"
                                            )
                                            or (self.write_mode != "static_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "update_statement"
                                            )
                                            or (self.write_mode != "update_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )

                    )

            )
            else exclude.add("create_statement")
        )
        (
            include.add("update_statement")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "update_statement")
                                    or (self.write_mode == "update_statement")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value == "update_statement_table_action"
                                    )
                                    or (self.write_mode == "update_statement_table_action")
                                )

                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value
                                        and "#" in str(self.write_mode.value)
                                    )
                                    or ("#" in str(self.write_mode))
                                )

                        )
                    )

            )
            else exclude.add("update_statement")
        )
        (
            include.add("schema_name")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (

                            (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "call_statement"
                                            )
                                            or (self.write_mode != "call_statement")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )
                                    )
                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "static_statement"
                                            )
                                            or (self.write_mode != "static_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "update_statement"
                                            )
                                            or (self.write_mode != "update_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )

                    )

            )
            else exclude.add("schema_name")
        )
        (
            include.add("execute_procedure_for_each_row")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        ((not self.update_statement)) or (self.update_statement and "#" in str(self.update_statement))
                    )
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call")
                                    or (self.write_mode == "call")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call_statement")
                                    or (self.write_mode == "call_statement")
                                )

                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value
                                        and "#" in str(self.write_mode.value)
                                    )
                                    or ("#" in str(self.write_mode))
                                )

                        )
                    )

            )
            else exclude.add("execute_procedure_for_each_row")
        )
        (
            include.add("table_name")
            if (

                    (((not self.static_statement)) or ((self.static_statement and "#" in str(self.static_statement))))
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (

                            (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (hasattr(self.write_mode, "value") and self.write_mode.value != "call")
                                            or (self.write_mode != "call")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )
                                    )
                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "call_statement"
                                            )
                                            or (self.write_mode != "call_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "static_statement"
                                            )
                                            or (self.write_mode != "static_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )
                            and (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value != "update_statement"
                                            )
                                            or (self.write_mode != "update_statement")
                                        )
                                    )
                                )
                                or (

                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )

                                )
                            )

                    )

            )
            else exclude.add("table_name")
        )
        (
            include.add("key_column_names")
            if (

                    (
                        (
                            (
                                ((not self.update_statement))
                                or ((self.update_statement and "#" in str(self.update_statement)))
                            )
                            and (
                                ((not self.static_statement))
                                or ((self.static_statement and "#" in str(self.static_statement)))
                            )
                        )
                    )
                    and (
                        (
                            (
                                (
                                    self.existing_table_action
                                    and (
                                        (
                                            hasattr(self.existing_table_action, "value")
                                            and self.existing_table_action.value == "append"
                                        )
                                        or (self.existing_table_action == "append")
                                    )
                                )
                            )
                            or (
                                (
                                    self.existing_table_action
                                    and (
                                        (
                                            hasattr(self.existing_table_action, "value")
                                            and self.existing_table_action.value == "merge"
                                        )
                                        or (self.existing_table_action == "merge")
                                    )
                                )
                            )
                            or (
                                (
                                    self.existing_table_action
                                    and (
                                        (
                                            hasattr(self.existing_table_action, "value")
                                            and self.existing_table_action.value == "update"
                                        )
                                        or (self.existing_table_action == "update")
                                    )
                                )
                            )
                        )
                        or (
                            (
                                (
                                    self.write_mode
                                    and (
                                        (hasattr(self.write_mode, "value") and self.write_mode.value == "insert")
                                        or (self.write_mode == "insert")
                                    )
                                )
                            )
                            or (

                                    self.write_mode
                                    and (
                                        (hasattr(self.write_mode, "value") and self.write_mode.value == "merge")
                                        or (self.write_mode == "merge")
                                    )

                            )
                            or (

                                    self.write_mode
                                    and (
                                        (hasattr(self.write_mode, "value") and self.write_mode.value == "update")
                                        or (self.write_mode == "update")
                                    )

                            )
                            or (

                                    self.write_mode
                                    and (
                                        (
                                            hasattr(self.write_mode, "value")
                                            and self.write_mode.value
                                            and "#" in str(self.write_mode.value)
                                        )
                                        or ("#" in str(self.write_mode))
                                    )

                            )
                        )
                    )

            )
            else exclude.add("key_column_names")
        )
        (
            include.add("table_action")
            if (

                    (
                        (
                            (
                                (
                                    (
                                        self.write_mode
                                        and (
                                            (hasattr(self.write_mode, "value") and self.write_mode.value == "insert")
                                            or (self.write_mode == "insert")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (hasattr(self.write_mode, "value") and self.write_mode.value == "merge")
                                            or (self.write_mode == "merge")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value == "update_statement_table_action"
                                            )
                                            or (self.write_mode == "update_statement_table_action")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.write_mode
                                        and (
                                            (
                                                hasattr(self.write_mode, "value")
                                                and self.write_mode.value
                                                and "#" in str(self.write_mode.value)
                                            )
                                            or ("#" in str(self.write_mode))
                                        )
                                    )
                                )
                            )
                        )
                    )
                    and (

                            (
                                ((not self.static_statement))
                                or ((self.static_statement and "#" in str(self.static_statement)))
                            )
                            and (

                                    (
                                        (
                                            (
                                                self.write_mode
                                                and (
                                                    (
                                                        hasattr(self.write_mode, "value")
                                                        and self.write_mode.value != "static_statement"
                                                    )
                                                    or (self.write_mode != "static_statement")
                                                )
                                            )
                                        )
                                        or (
                                            (
                                                self.write_mode
                                                and (
                                                    (
                                                        hasattr(self.write_mode, "value")
                                                        and self.write_mode.value
                                                        and "#" in str(self.write_mode.value)
                                                    )
                                                    or ("#" in str(self.write_mode))
                                                )
                                            )
                                        )
                                    )
                                    and (
                                        (
                                            (
                                                self.write_mode
                                                and (
                                                    (
                                                        hasattr(self.write_mode, "value")
                                                        and self.write_mode.value != "update_statement"
                                                    )
                                                    or (self.write_mode != "update_statement")
                                                )
                                            )
                                        )
                                        or (

                                                self.write_mode
                                                and (
                                                    (
                                                        hasattr(self.write_mode, "value")
                                                        and self.write_mode.value
                                                        and "#" in str(self.write_mode.value)
                                                    )
                                                    or ("#" in str(self.write_mode))
                                                )

                                        )
                                    )

                            )

                    )

            )
            else exclude.add("table_action")
        )
        (
            include.add("static_statement")
            if (

                    (((not self.schema_name)) or ((self.schema_name and "#" in str(self.schema_name))))
                    and (((not self.table_name)) or (self.table_name and "#" in str(self.table_name)))
                    and (
                        ((not self.update_statement)) or (self.update_statement and "#" in str(self.update_statement))
                    )
                    and (
                        ((not self.stored_procedure_name))
                        or (self.stored_procedure_name and "#" in str(self.stored_procedure_name))
                    )
                    and (
                        ((not self.call_procedure_statement))
                        or (self.call_procedure_statement and "#" in str(self.call_procedure_statement))
                    )
                    and (
                        (
                            (
                                self.write_mode
                                and (
                                    (hasattr(self.write_mode, "value") and self.write_mode.value == "static_statement")
                                    or (self.write_mode == "static_statement")
                                )
                            )
                        )
                        or (

                                self.write_mode
                                and (
                                    (
                                        hasattr(self.write_mode, "value")
                                        and self.write_mode.value
                                        and "#" in str(self.write_mode.value)
                                    )
                                    or ("#" in str(self.write_mode))
                                )

                        )
                    )

            )
            else exclude.add("static_statement")
        )
        (
            include.add("runtime_column_propagation")
            if (not self.enable_schemaless_design)
            else exclude.add("runtime_column_propagation")
        )
        include.add("data_asset_name") if (self.create_data_asset) else exclude.add("data_asset_name")
        include.add("create_data_asset") if (()) else exclude.add("create_data_asset")
        (
            include.add("schema_name")
            if (not self.static_statement)
            and (not self.call_procedure_statement)
            and (
                self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call_statement" not in str(self.write_mode.value)
                    )
                    or ("call_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "static_statement" not in str(self.write_mode.value)
                    )
                    or ("static_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "update_statement" not in str(self.write_mode.value)
                    )
                    or ("update_statement" not in str(self.write_mode))
                )
            )
            else exclude.add("schema_name")
        )
        (
            include.add("stored_procedure_name")
            if (not self.static_statement)
            and (not self.table_name)
            and (not self.update_statement)
            and (not self.call_procedure_statement)
            and (
                self.write_mode
                and (
                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call")
                    or (self.write_mode == "call")
                )
            )
            else exclude.add("stored_procedure_name")
        )
        (
            include.add("key_column_names")
            if ((not self.update_statement) and (not self.static_statement))
            and (
                (
                    self.existing_table_action
                    and (
                        (
                            hasattr(self.existing_table_action, "value")
                            and self.existing_table_action.value
                            and "append" in str(self.existing_table_action.value)
                        )
                        or ("append" in str(self.existing_table_action))
                    )
                    or self.existing_table_action
                    and (
                        (
                            hasattr(self.existing_table_action, "value")
                            and self.existing_table_action.value
                            and "merge" in str(self.existing_table_action.value)
                        )
                        or ("merge" in str(self.existing_table_action))
                    )
                    or self.existing_table_action
                    and (
                        (
                            hasattr(self.existing_table_action, "value")
                            and self.existing_table_action.value
                            and "update" in str(self.existing_table_action.value)
                        )
                        or ("update" in str(self.existing_table_action))
                    )
                )
                or (
                    self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "insert" in str(self.write_mode.value)
                        )
                        or ("insert" in str(self.write_mode))
                    )
                    or self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "merge" in str(self.write_mode.value)
                        )
                        or ("merge" in str(self.write_mode))
                    )
                    or self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "update" in str(self.write_mode.value)
                        )
                        or ("update" in str(self.write_mode))
                    )
                )
            )
            else exclude.add("key_column_names")
        )
        (
            include.add("create_statement")
            if (not self.static_statement)
            and (not self.stored_procedure_name)
            and (not self.call_procedure_statement)
            and (
                self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call" not in str(self.write_mode.value)
                    )
                    or ("call" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call_statement" not in str(self.write_mode.value)
                    )
                    or ("call_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "static_statement" not in str(self.write_mode.value)
                    )
                    or ("static_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "update_statement" not in str(self.write_mode.value)
                    )
                    or ("update_statement" not in str(self.write_mode))
                )
            )
            else exclude.add("create_statement")
        )
        (
            include.add("static_statement")
            if (not self.schema_name)
            and (not self.table_name)
            and (not self.update_statement)
            and (not self.stored_procedure_name)
            and (not self.call_procedure_statement)
            and (
                self.write_mode
                and (
                    (hasattr(self.write_mode, "value") and self.write_mode.value == "static_statement")
                    or (self.write_mode == "static_statement")
                )
            )
            else exclude.add("static_statement")
        )
        (
            include.add("table_name")
            if (not self.static_statement)
            and (not self.call_procedure_statement)
            and (
                self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call" not in str(self.write_mode.value)
                    )
                    or ("call" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call_statement" not in str(self.write_mode.value)
                    )
                    or ("call_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "static_statement" not in str(self.write_mode.value)
                    )
                    or ("static_statement" not in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "update_statement" not in str(self.write_mode.value)
                    )
                    or ("update_statement" not in str(self.write_mode))
                )
            )
            else exclude.add("table_name")
        )
        (
            include.add("batch_size")
            if (not self.has_reject_output) or (self.has_reject_output != "true" or not self.has_reject_output)
            else exclude.add("batch_size")
        )
        (
            include.add("update_statement")
            if (not self.static_statement)
            and (
                self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "update_statement" in str(self.write_mode.value)
                    )
                    or ("update_statement" in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "update_statement_table_action" in str(self.write_mode.value)
                    )
                    or ("update_statement_table_action" in str(self.write_mode))
                )
            )
            else exclude.add("update_statement")
        )
        (
            include.add("call_procedure_statement")
            if (not self.static_statement)
            and (not self.schema_name)
            and (not self.table_name)
            and (not self.update_statement)
            and (not self.stored_procedure_name)
            and (
                self.write_mode
                and (
                    (hasattr(self.write_mode, "value") and self.write_mode.value == "call_statement")
                    or (self.write_mode == "call_statement")
                )
            )
            else exclude.add("call_procedure_statement")
        )
        (
            include.add("table_action")
            if (

                    self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "insert" in str(self.write_mode.value)
                        )
                        or ("insert" in str(self.write_mode))
                    )
                    and self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "merge" in str(self.write_mode.value)
                        )
                        or ("merge" in str(self.write_mode))
                    )
                    and self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "update_statement_table_action" in str(self.write_mode.value)
                        )
                        or ("update_statement_table_action" in str(self.write_mode))
                    )

            )
            and (
                (not self.static_statement)
                and (
                    self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "static_statement" not in str(self.write_mode.value)
                        )
                        or ("static_statement" not in str(self.write_mode))
                    )
                    and self.write_mode
                    and (
                        (
                            hasattr(self.write_mode, "value")
                            and self.write_mode.value
                            and "update_statement" not in str(self.write_mode.value)
                        )
                        or ("update_statement" not in str(self.write_mode))
                    )
                )
            )
            else exclude.add("table_action")
        )
        (
            include.add("execute_procedure_for_each_row")
            if (not self.static_statement)
            and (not self.table_name)
            and (not self.update_statement)
            and (
                self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call" in str(self.write_mode.value)
                    )
                    or ("call" in str(self.write_mode))
                )
                and self.write_mode
                and (
                    (
                        hasattr(self.write_mode, "value")
                        and self.write_mode.value
                        and "call_statement" in str(self.write_mode.value)
                    )
                    or ("call_statement" in str(self.write_mode))
                )
            )
            else exclude.add("execute_procedure_for_each_row")
        )
        return include, exclude

    def get_source_props(self) -> dict:
        include, exclude = self.validate_source()
        props = {
            "add_procedure_return_value_to_schema",
            "buf_free_run_ronly",
            "buf_mode_ronly",
            "buffer_free_run_percent",
            "buffering_mode",
            "byte_limit",
            "call_procedure_statement",
            "collecting",
            "column_metadata_change_propagation",
            "combinability_mode",
            "credentials_input_method_ssl",
            "current_output_link_type",
            "db2_database_name",
            "db2_instance_name",
            "db2_source_connection_required",
            "db2_table_name",
            "decimal_rounding_mode",
            "default_maximum_length_for_columns",
            "disk_write_inc_ronly",
            "disk_write_increment_bytes",
            "ds_java_heap_size",
            "enable_after_sql",
            "enable_after_sql_node",
            "enable_before_sql",
            "enable_before_sql_node",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "error_warning",
            "execution_mode",
            "fail_on_error_after_sql",
            "fail_on_error_after_sql_node",
            "fail_on_error_before_sql",
            "fail_on_error_before_sql_node",
            "fatal_error",
            "flow_dirty",
            "forward_row_data",
            "generate_unicode_type_columns",
            "hide",
            "infer_schema",
            "input_count",
            "input_link_description",
            "input_method",
            "inputcol_properties",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "key_column_names",
            "max_mem_buf_size_ronly",
            "maximum_memory_buffer_size_bytes",
            "output_acp_should_hide",
            "output_count",
            "output_link_description",
            "outputcol_properties",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "partition_type",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "preserve_partitioning",
            "proc_param_properties",
            "push_filters",
            "pushed_filters",
            "query_timeout",
            "queue_upper_bound_size_bytes",
            "queue_upper_size_ronly",
            "read_method",
            "rejected_filters",
            "row_limit",
            "runtime_column_propagation",
            "schema_name",
            "select_statement",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "sorting_key",
            "stable",
            "stage_description",
            "stored_procedure_name",
            "table_name",
            "transform",
            "unique",
            "user_defined_function",
        }
        required = {
            "access_token",
            "alternate_hostname",
            "alternate_port",
            "current_output_link_type",
            "database",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "hostname_or_ip_address",
            "output_acp_should_hide",
            "password",
            "port",
            "select_statement",
            "table_name",
            "username",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_target_props(self) -> dict:
        include, exclude = self.validate_target()
        props = {
            "batch_size",
            "buf_free_run_ronly",
            "buf_mode_ronly",
            "buffer_free_run_percent",
            "buffering_mode",
            "call_procedure_statement",
            "collecting",
            "column_metadata_change_propagation",
            "combinability_mode",
            "create_data_asset",
            "create_statement",
            "credentials_input_method_ssl",
            "current_output_link_type",
            "data_asset_name",
            "db2_database_name",
            "db2_instance_name",
            "db2_source_connection_required",
            "db2_table_name",
            "disk_write_inc_ronly",
            "disk_write_increment_bytes",
            "ds_java_heap_size",
            "enable_after_sql",
            "enable_after_sql_node",
            "enable_before_sql",
            "enable_before_sql_node",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "execute_procedure_for_each_row",
            "execution_mode",
            "existing_table_action",
            "fail_on_error_after_sql",
            "fail_on_error_after_sql_node",
            "fail_on_error_before_sql",
            "fail_on_error_before_sql_node",
            "flow_dirty",
            "has_reject_output",
            "hide",
            "input_count",
            "input_link_description",
            "input_method",
            "inputcol_properties",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "key_column_names",
            "max_mem_buf_size_ronly",
            "maximum_memory_buffer_size_bytes",
            "output_acp_should_hide",
            "output_count",
            "output_link_description",
            "outputcol_properties",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "partition_type",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "preserve_partitioning",
            "proc_param_properties",
            "query_timeout",
            "queue_upper_bound_size_bytes",
            "queue_upper_size_ronly",
            "runtime_column_propagation",
            "schema_name",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "sorting_key",
            "stable",
            "stage_description",
            "static_statement",
            "stored_procedure_name",
            "table_action",
            "table_name",
            "unique",
            "update_statement",
            "write_mode",
        }
        required = {
            "access_token",
            "alternate_hostname",
            "alternate_port",
            "current_output_link_type",
            "data_asset_name",
            "database",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "hostname_or_ip_address",
            "output_acp_should_hide",
            "password",
            "port",
            "static_statement",
            "table_name",
            "username",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_parameters_props(self) -> dict:
        include, exclude = self.validate_parameters()
        props = {"execution_mode", "input_count", "output_count", "preserve_partitioning"}
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_app_data_props(self) -> dict:
        return {
            "datastage": {
                "active": 0,
                "SupportsRef": True,
                "maxRejectOutputs": 1,
                "minRejectOutputs": 0,
                "maxReferenceInputs": 0,
                "minReferenceInputs": 0,
            }
        }

    def get_input_ports_props(self) -> dict:
        include, exclude = self.validate_target()
        props = {"runtime_column_propagation"}
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_input_cardinality(self) -> dict:
        return {"min": 0, "max": 1}

    def get_output_ports_props(self) -> dict:
        include, exclude = self.validate_source()
        props = {
            "reject_condition_row_is_rejected",
            "reject_data_element_errorcode",
            "reject_data_element_errortext",
            "reject_from_link",
            "reject_number",
            "reject_threshold",
            "reject_uses",
        }
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_output_cardinality(self) -> dict:
        return {"min": 0, "max": 1}

    def get_allowed_as_source_props(self) -> bool:
        return True

    def get_allowed_as_target_props(self) -> bool:
        return True



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025


"""This module contains the AccountAPIClient class."""

import requests
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_ACCOUNT_API_VERSION = 2


class AccountAPIClient(BaseAPIClient):
    """The API client of the Accounts service."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = "https://accounts.cloud.ibm.com",
    ) -> None:
        """Initializes the AccountAPIClient.

        Args:
            auth: The Authentication object.
            base_url: The Accounts Service URL.
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path = f"v{DEFAULT_ACCOUNT_API_VERSION}/accounts"

    def get_accounts(self, params: dict = None) -> requests.Response:
        """Retrieve a list of accounts associated with the current IAM bearer token.

        Args:
            params: Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/coe/{self.url_path}"
        response = self.get(url=url, params=params)
        return response

    def get_account(self, account_id: str) -> requests.Response:
        """Retrieve details of a specific account by its ID.

        Args:
            account_id: Unique identifier of the account.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/coe/{self.url_path}/{quote(account_id, safe='')}"
        response = self.get(url=url)
        return response



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module containing the AccessGroupsAPIClient class."""

import requests
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_ACCESS_GROUP_API_VERSION = 2


class AccessGroupsAPIClient(BaseAPIClient):
    """The API Client of Access Groups."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = "https://iam.cloud.ibm.com",
    ) -> None:
        """Initializes the AccessGroupsAPIClient.

        Args:
            auth: The Authentication object.
            base_url: Default is "https://iam.cloud.ibm.com".
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path = f"v{DEFAULT_ACCESS_GROUP_API_VERSION}/groups"

    def get_access_group(self, access_group_id: str) -> requests.Response:
        """Gets an access group.

        Args:
            access_group_id: The access group ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}"
        response = self.get(url=url)
        return response

    def get_all_access_groups(self, params: dict) -> requests.Response:
        """Lists all access groups under an account. Will only list those the listed user can access.

        Args:
            params: Query parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}"
        response = self.get(url=url, params=params)
        return response

    def create_access_group(self, params: dict, data: dict = None) -> requests.Response:
        """Creates a new access group.

        Args:
            params: Query parameters.
            data: The name and description of the access group to be created/updated, in json form.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}"
        response = self.post(url=url, params=params, data=data)
        return response

    def update_access_group(self, access_group_id: str, etag: str, data: dict) -> requests.Response:
        """Updates existing access group.

        Args:
            access_group_id: The access group ID.
            etag: The etag for the latest revision to the AccessGroup.
            data: The name and description of the access group to be created/updated, in json form.

        Returns:
            A HTTP response.
        """
        headers = {"If-Match": etag}

        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}"
        response = self.patch(url=url, data=data, headers=headers)
        return response

    def delete_access_group(self, access_group_id: str) -> requests.Response:
        """Deletes an access group from an account.

        Args:
            access_group_id: The access group ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}"
        response = self.delete(url=url)
        return response

    def add_members(self, access_group_id: str, data: dict) -> requests.Response:
        """Adds member(s) to an access group.

        Args:
            access_group_id: The access group ID.
            data: The member(s) to be added to an access group, in json form.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}/members"
        response = self.put(url, data=data)
        return response

    def get_members(self, access_group_id: str) -> requests.Response:
        """Lists all members in an access group.

        Args:
            access_group_id: The access group ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}/members"
        response = self.get(url)
        return response

    def check_membership(self, access_group_id: str, iam_id: str) -> requests.Response:
        """Checks if a member is present in an access group.

        Args:
            access_group_id: The access group ID.
            iam_id: The IAM ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}/members/{iam_id}"
        response = self.head(url)
        return response

    def remove_members_from_access_group(self, access_group_id: str, data: dict) -> requests.Response:
        """Removes multiple members from an access group.

        Args:
            access_group_id: The access group ID.
            data: List of IAM IDs to remove, in json form
        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(access_group_id, safe='')}/members/delete"
        response = self.post(url, data=data)
        return response

    def remove_member_from_all_access_groups(self, iam_id: str, params: dict) -> requests.Response:
        """Removes member from all access groups under an account.

        Args:
            params: Query parameters.
            iam_id: The IAM ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/_allgroups/members/{quote(iam_id, safe='')}"
        response = self.delete(url, params)
        return response

    def add_member_to_multiple_access_groups(self, params: dict, iam_id: str, data: dict) -> requests.Response:
        """Adds member to multiple access groups.

        Args:
            params: Query parameters.
            iam_id: The IAM ID.
            data: The type of member and the groups to add a member to, in json form.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/_allgroups/members/{quote(iam_id, safe='')}"
        response = self.put(url, params, data)
        return response



from ibm_watsonx_data_integration.services.datastage.models.enums import APACHE_KAFKA, APACHE_KAFKA_CONNECTION
from ibm_watsonx_data_integration.services.datastage.models.flow.datastage_flow import DataStageFlow
from ibm_watsonx_data_integration.services.datastage.tests.integration.util import AbstractFlowRunTest
from os import environ
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestApacheKafkaConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "apachekafka_peek"

    def get_fc(self, project: Project):
        fc = DataStageFlow(project=project, name=self.get_flow_name(), description="", flow_type="datastage")

        # Stages
        _uitesting_1_ = fc.add_stage("Apache Kafka", "uitesting_1")
        _uitesting_1_.configuration.consumer_group = "DataStage"
        _uitesting_1_.configuration.kafka_client_logging_level = APACHE_KAFKA.KafkaClientLoggingLevel.info
        _uitesting_1_.configuration.topic_name = "uitesting"
        _uitesting_1_.configuration.connection.password = environ["APACHEKAFKA_PASSWORD"]
        _uitesting_1_.configuration.connection.secure_connection = APACHE_KAFKA_CONNECTION.SecureConnection.SASL_SSL
        _uitesting_1_.configuration.connection.kafka_server_host_name = environ["APACHEKAFKA_SERVER_NAME"]
        _uitesting_1_.configuration.connection.use_schema_registry_for_message_format = False
        _uitesting_1_.configuration.connection.username = environ["APACHEKAFKA_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")
        _peek_1_.configuration.runtime_column_propagation = 0

        # Graph
        uitesting_1_schema = _uitesting_1_.connect_output_to(_peek_1_).set_name("Link_2").create_schema()

        # Schemas
        uitesting_1_schema.add_field("CHAR", "COLUMN_1").length(100)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state
